@@ -41,6 +41,7 @@
 #include "file-utils.h"
 #include "fr-process.h"
 #include "gconf-utils.h"
+#include "glib-utils.h"
 #include "dlg-prop.h"
 
 
@@ -87,253 +88,6 @@ new_archive (GtkWidget *file_sel,
 }
 
 
-typedef struct {
-	char *name;
-	char *ext;
-} FileTypeDescription;
-
-
-/* these arrays need to increase in size if new types are introduced */
-
-static FileTypeDescription write_type_desc[18]; 
-static char * save_mime_type[19];
-static char *open_mime_type[24];
-
-
-void 
-get_supported_archive_types (void)
-{
-	int  wi, si, oi;
-
-	wi = 0;
-	si = 0;
-	oi = 0;
-
-	write_type_desc[wi].name = g_strdup(N_("Automatic"));
-	write_type_desc[wi].ext = NULL;
-	wi++;
-
-	if (is_program_in_path ("ar")) {
-		write_type_desc[wi].name = g_strdup(N_("Ar (.ar)"));
-		write_type_desc[wi].ext = g_strdup(".ar");
-		wi++;
-		save_mime_type[si] = g_strdup("application/x-ar");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-ar");
-		oi++;
-		open_mime_type[oi] = g_strdup("application/x-deb");
-		oi++;
-	}
-
-	if (is_program_in_path ("arj")) {
-                write_type_desc[wi].name = g_strdup(N_("Arj (.arj)"));
-		write_type_desc[wi].ext = g_strdup(".arj");	
-                wi++;
-		save_mime_type[si] = g_strdup("application/x-arj");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-arj");
-                oi++;
-        }
-
-	if (is_program_in_path ("bzip2")) {
-                save_mime_type[si] = g_strdup("application/x-bzip");
-                si++;
-		open_mime_type[oi] = g_strdup("application/x-bzip");
-		oi++;
-        }
-
-	if (is_program_in_path ("compress")) {
-		save_mime_type[si] = g_strdup("application/x-compress");
-                si++;
-        }
-
-	if (is_program_in_path ("isoinfo")) {
-		save_mime_type[si] = g_strdup("application/x-cd-image");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-cd-image");
-		oi++;
-        }
-
-	if (is_program_in_path ("zip")) {
-                write_type_desc[wi].name = g_strdup(N_("Ear (.ear)"));
-                write_type_desc[wi].ext = g_strdup(".ear");
-                wi++;
-	}
-
-	if (is_program_in_path ("gzip")) {
-                save_mime_type[si] = g_strdup("application/x-gzip");
-                si++;
-		open_mime_type[oi] = g_strdup("application/x-gzip");
-		oi++;
-        }
-
-	if (is_program_in_path ("zip")) {
-                write_type_desc[wi].name = g_strdup(N_("Jar (.jar)"));
-		write_type_desc[wi].ext = g_strdup(".jar");
-                wi++;
-		save_mime_type[si] = g_strdup("application/x-jar");
-		si++;
-		save_mime_type[si] = g_strdup("application/x-java-archive");
-                si++;
-		open_mime_type[oi] = g_strdup("application/x-java-archive");
-		oi++;
-		open_mime_type[oi] = g_strdup("application/x-jar");
-		oi++;
-        }
-
-	if (is_program_in_path ("lha")) {
-                write_type_desc[wi].name = g_strdup(N_("Lha (.lzh)"));
-		write_type_desc[wi].ext = g_strdup(".lzh");
-                wi++;
-		save_mime_type[si++] = g_strdup("application/x-lha");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-lha");
-		oi++;
-        }
-
-	if (is_program_in_path ("lzop")) {
-                save_mime_type[si] = g_strdup("application/x-lzop");
-                si++;
-		open_mime_type[oi] = g_strdup("application/x-lzop");
-                oi++;
-        }
-
-	if (is_program_in_path ("rar")) {
-                write_type_desc[wi].name = g_strdup(N_("Rar (.rar)"));
-		write_type_desc[wi].ext = g_strdup(".rar");
-                wi++;
-		save_mime_type[si] = g_strdup("application/x-rar");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-rar");
-		oi++;
-        }
-
-	if (is_program_in_path ("rpm2cpio")) {
-                open_mime_type[oi] = g_strdup("application/x-rpm");
-                oi++;
-        }
-
-	if (is_program_in_path ("tar")) {
-                write_type_desc[wi].name = g_strdup(N_("Tar uncompressed (.tar)"));
-		write_type_desc[wi].ext = g_strdup(".tar");
-                wi++;
-		save_mime_type[si] = g_strdup("application/x-tar");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-tar");
-		oi++;
-
-		if (is_program_in_path ("bzip")) {
-                        write_type_desc[wi].name = g_strdup(N_("Tar compressed with bzip (.tar.bz)"));
-                        write_type_desc[wi].ext = g_strdup(".tar.bz");
-                        wi++;
-                }
-
-		if (is_program_in_path ("bzip2")) {
-                        write_type_desc[wi].name = g_strdup(N_("Tar compressed with bzip2 (.tar.bz2)"));
-                        write_type_desc[wi].ext = g_strdup(".tar.bz2");
-                        wi++;
-			save_mime_type[si] = g_strdup("application/x-bzip-compressed-tar");
-                        si++;
-			open_mime_type[oi] = g_strdup("application/x-bzip-compressed-tar");
-			oi++;
-                }
-
-		if (is_program_in_path ("gzip")) {
-                        write_type_desc[wi].name = g_strdup(N_("Tar compressed with gzip (.tar.gz)"));
-                        write_type_desc[wi].ext = g_strdup(".tar.gz");
-                        wi++;
-			save_mime_type[si] = g_strdup("application/x-compressed-tar");
-			si++;
-			open_mime_type[oi] = g_strdup("application/x-compressed-tar");
-			oi++;
-                }
-
-		if (is_program_in_path ("lzop")) {
-                        write_type_desc[wi].name = g_strdup(N_("Tar compressed with lzop (.tar.lzo)"));
-                        write_type_desc[wi].ext = g_strdup(".tar.lzo");
-                        wi++;
-			open_mime_type[oi] = g_strdup("application/x-lzop-compressed-tar");	
-			oi++;
-                }
-
-		if (is_program_in_path ("compress")) {
-                        write_type_desc[wi].name = g_strdup(N_("Tar compressed with compress (.tar.Z)"));
-                        write_type_desc[wi].ext = g_strdup(".tar.Z");
-                        wi++;
-                }
-
-		if (is_program_in_path ("rar")) {
-                	save_mime_type[si] = g_strdup("application/x-rar-compressed");
-                	si++;
-                	open_mime_type[oi] = g_strdup("application/x-rar-compressed");
-                	oi++;
-        	}
-	}		
-
-	if (is_program_in_path ("uncompress")) {
-                open_mime_type[oi] = g_strdup("application/x-compress");
-                oi++;
-        }
-
-	if (is_program_in_path ("unstuff")) {
-                open_mime_type[oi] = g_strdup("application/x-stuffit");
-                oi++;
-        }
-
-	if (is_program_in_path ("unzip")) {
-                open_mime_type[oi] = g_strdup("application/zip");
-                oi++;
-                open_mime_type[oi] = g_strdup("application/x-zip");
-                oi++;
-        }
-        
-	if (is_program_in_path ("zip")) {
-                write_type_desc[wi].name = g_strdup(N_("War (.war)"));
-                write_type_desc[wi].ext = g_strdup(".war");
-                wi++;
-		write_type_desc[wi].name = g_strdup(N_("Zip (.zip)"));
-                write_type_desc[wi].ext = g_strdup(".zip");
-                wi++;
-		save_mime_type[si] = g_strdup("application/zip");
-                si++;
-        }
-
-	if (is_program_in_path ("zoo")) {
-                write_type_desc[wi].name = g_strdup(N_("Zoo (.zoo)"));
-		write_type_desc[wi].ext = g_strdup(".7z");
-                wi++;
-		save_mime_type[si] = g_strdup("application/x-zoo");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-zoo");
-		oi++;
-        }
-
-	if (is_program_in_path ("7za")) {
-                write_type_desc[wi].name = g_strdup(N_("7-Zip (.7z)"));
-		write_type_desc[wi].ext = g_strdup(".zoo");
-                wi++;
-		save_mime_type[si] = g_strdup("application/x-7zip");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-7zip");
-		oi++;	
-
-        } else if (is_program_in_path ("7z")) {
-		write_type_desc[wi].name = g_strdup(N_("7-Zip (.7z)"));
-		write_type_desc[wi].ext = g_strdup(".7z");
-		wi++;
-		save_mime_type[si] = g_strdup("application/x-7zip");
-		si++;
-		open_mime_type[oi] = g_strdup("application/x-7zip");
-		oi++;
-	}
-
-	write_type_desc[wi].name = NULL;
-	write_type_desc[wi].ext = NULL;
-	save_mime_type[si] = NULL;
-	open_mime_type[oi] = NULL;
-}
-
-
 /* when on Automatic the user provided extension needs to be supported, 
    otherwise an existing unsupported archive can be deleted (if the user
    provided name matches with its name) before we find out that the 
@@ -343,18 +97,10 @@ static gboolean
 is_supported_extension (GtkWidget *file_sel, 
 			char      *filename)
 {
-	int        i;
-	GtkWidget *combo_box;
-	
-	combo_box = g_object_get_data (G_OBJECT (file_sel), "fr_combo_box");
-	if (gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box)) > 0) 
-		/* not on Automatic */
-		return TRUE;
-	
-	for (i = 1; write_type_desc[i].name != NULL; i++) 
-		if (file_extension_is (filename, write_type_desc[i].ext)) 
+	int i;
+	for (i = 0; save_type[i] != FR_FILE_TYPE_NULL; i++) 
+		if (file_extension_is (filename, file_type_desc[save_type[i]].ext)) 
 			return TRUE;
-	
 	return FALSE;
 }
 
@@ -363,10 +109,10 @@ static char *
 get_full_path (GtkWidget *file_sel)
 {
 	GtkWidget   *combo_box;
-	char        *full_path;
+	char        *full_path = NULL;
 	char        *path;
 	const char  *filename;
-	int          file_type_idx;
+	int          idx;
 
 	combo_box = g_object_get_data (G_OBJECT (file_sel), "fr_combo_box");
 	path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel));
@@ -380,38 +126,30 @@ get_full_path (GtkWidget *file_sel)
 		return NULL;
 	}
 	
-	file_type_idx = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
-	if (file_type_idx > 0) {
-		full_path = g_strconcat (path, 
-					 write_type_desc[file_type_idx].ext,
-					 NULL);
-		g_free (path);
-	} else {
+	idx = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
+	if (idx > 0) {
+		const char *path_ext = fr_archive_utils__get_file_name_ext (path);
+		char       *default_ext = file_type_desc[save_type[idx-1]].ext;
+		if (strcmp_null_tollerant (path_ext, default_ext) != 0) {
+			full_path = g_strconcat (path, default_ext, NULL);
+			g_free (path);
+		}
+	} 
+	if (full_path == NULL)
 		full_path = path;
-	}
 
 	return full_path;
 }
 
 
-static void 
-new_file_response_cb (GtkWidget *w,
-		      int        response,
-		      GtkWidget *file_sel)
+static char *
+get_archive_filename_from_selector (FRWindow  *window,
+				    GtkWidget *file_sel)
 {
-	FRWindow *window;
-	gchar    *path;
-	gchar    *dir;
-
-	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
-		gtk_widget_destroy (file_sel);
-		return;
-	}
-
-	window = g_object_get_data (G_OBJECT (file_sel), "fr_window");
+	char *path = NULL;
+	char *dir;
 
 	path = get_full_path (file_sel);
-
 	if ((path == NULL) || (*path == 0)) {
 		GtkWidget *dialog;
 
@@ -427,7 +165,7 @@ new_file_response_cb (GtkWidget *w,
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
-		return;
+		return NULL;
 	}
 	
 	dir = remove_level_from_path (path);
@@ -448,27 +186,36 @@ new_file_response_cb (GtkWidget *w,
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
-		return;
+		return NULL;
 	}
 	g_free (dir);
 
-	/* if the user do not specify an extension use tgz as default */
-	if (strchr (path, '.') == NULL) {
-		char *new_path;
-		new_path = g_strconcat (path, ".tgz", NULL);
+	/* if the user did not specify a valid extension use the filetype combobox current type
+	 * or tar.gz if automatic is selected. */
+	if (fr_archive_utils__get_file_name_ext (path) == NULL) {
+		GtkWidget *combo_box;
+		int        idx;
+		char      *new_path;
+		char      *ext = NULL;
+
+		combo_box = g_object_get_data (G_OBJECT (file_sel), "fr_combo_box");
+		idx = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
+		if (idx > 0)
+			ext = file_type_desc[save_type[idx-1]].ext;
+		else
+			ext = ".tar.gz";
+		new_path = g_strconcat (path, ext, NULL);
 		g_free (path);
 		path = new_path;
 	}
 
-#ifdef DEBUG
-	g_print ("create %s\n", path); 
-#endif
+	debug (DEBUG_INFO, "create/save %s\n", path); 
 
 	if (path_is_file (path)) {
 		GtkWidget *dialog;
 		int        r;
 
-		if (!is_supported_extension(file_sel, path)) {
+		if (!is_supported_extension (file_sel, path)) {
                 	dialog = _gtk_message_dialog_new (GTK_WINDOW (file_sel),
                                                   GTK_DIALOG_MODAL,
                                                   GTK_STOCK_DIALOG_ERROR,
@@ -480,7 +227,7 @@ new_file_response_cb (GtkWidget *w,
                 	gtk_dialog_run (GTK_DIALOG (dialog));
                 	gtk_widget_destroy (GTK_WIDGET (dialog));
 			g_free (path);
-			return;
+			return NULL;
 		}
 
 		dialog = _gtk_message_dialog_new (GTK_WINDOW (file_sel),
@@ -498,7 +245,7 @@ new_file_response_cb (GtkWidget *w,
 
 		if (r != GTK_RESPONSE_YES) {
 			g_free (path);
-			return;
+			return NULL;
 		}
 
 		if (unlink (path) != 0) {
@@ -514,12 +261,70 @@ new_file_response_cb (GtkWidget *w,
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (GTK_WIDGET (dialog));
 			g_free (path);
-			return;
+			return NULL;
 		}
-	} 
+	}
 
-	new_archive (file_sel, window, path);
-	g_free (path);
+	return path;
+}
+
+
+static void 
+new_file_response_cb (GtkWidget *w,
+		      int        response,
+		      GtkWidget *file_sel)
+{
+	FRWindow *window;
+	char     *path;
+
+	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
+		gtk_widget_destroy (file_sel);
+		return;
+	}
+
+	window = g_object_get_data (G_OBJECT (file_sel), "fr_window");
+
+	path = get_archive_filename_from_selector (window, file_sel);
+	if (path != NULL) {
+		new_archive (file_sel, window, path);
+		g_free (path);
+	}
+}
+
+
+static void
+filetype_combobox_changed_cb (GtkComboBox *combo_box,
+			      GtkWidget   *file_sel)
+{
+	int         idx;
+	const char *filename;
+	const char *ext, *newext;
+	char       *new_filename, *filename_noext;
+
+	idx = gtk_combo_box_get_active (combo_box) - 1;
+	if (idx < 0) 
+		return;
+
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel));
+	if (filename == NULL)
+		return;
+
+	filename = file_name_from_path (filename);
+	if (filename == NULL)
+		return;
+
+	ext = fr_archive_utils__get_file_name_ext (filename);
+	if (ext == NULL)
+		ext = "";
+
+	filename_noext = g_strndup (filename, strlen (filename) - strlen (ext));
+
+	newext = file_type_desc[save_type[idx]].ext;
+	new_filename = g_strconcat (filename_noext, newext, NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_sel), new_filename);
+
+	g_free (new_filename);
+	g_free (filename_noext);
 }
 
 
@@ -527,12 +332,12 @@ void
 activate_action_new (GtkAction *action, 
 		     gpointer   data)
 {
-	FRWindow  *window = data;
-	GtkWidget *file_sel;
-	GtkWidget *hbox;
-	GtkWidget *combo_box;
+	FRWindow      *window = data;
+	GtkWidget     *file_sel;
+	GtkWidget     *hbox;
+	GtkWidget     *combo_box;
 	GtkFileFilter *filter;
-	int i;
+	int            i;
 
 	file_sel = gtk_file_chooser_dialog_new (
 			_("New"),
@@ -573,8 +378,8 @@ activate_action_new (GtkAction *action,
 
 	filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (filter, _("All archives"));
-	for (i = 0; save_mime_type[i] != NULL; i++)
-		gtk_file_filter_add_mime_type (filter, save_mime_type[i]);
+	for (i = 0; save_type[i] != FR_FILE_TYPE_NULL; i++) 
+		gtk_file_filter_add_mime_type (filter, file_type_desc[save_type[i]].mime_type);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_sel), filter);
 
@@ -593,9 +398,10 @@ activate_action_new (GtkAction *action,
 			    FALSE, FALSE, 0);
 
 	combo_box = gtk_combo_box_new_text ();
-	for (i = 0; write_type_desc[i].name != NULL; i++)
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), _("Automatic"));
+	for (i = 0; save_type[i] != FR_FILE_TYPE_NULL; i++)
 		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
-					   _(write_type_desc[i].name));
+					   _(file_type_desc[save_type[i]].name));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
 	gtk_box_pack_start (GTK_BOX (hbox), combo_box, TRUE, TRUE, 0);
 	gtk_widget_show_all (hbox);
@@ -610,11 +416,14 @@ activate_action_new (GtkAction *action,
 			  "response", 
 			  G_CALLBACK (new_file_response_cb), 
 			  file_sel);
-
 	g_signal_connect (G_OBJECT (file_sel),
 			  "destroy", 
 			  G_CALLBACK (new_file_destroy_cb),
 			  file_sel);
+	g_signal_connect (G_OBJECT (combo_box),
+			  "changed", 
+			  G_CALLBACK (filetype_combobox_changed_cb),
+			  file_sel);
 
 	gtk_window_set_modal (GTK_WINDOW (file_sel),TRUE);
 	gtk_widget_show_all (file_sel);
@@ -633,11 +442,11 @@ open_file_destroy_cb (GtkWidget *w,
 
 static void 
 open_file_response_cb (GtkWidget *w,
-		       gint response,
+		       int        response,
 		       GtkWidget *file_sel)
 {
-	FRWindow   *window = NULL;
-	char *path;
+	FRWindow *window = NULL;
+	char     *path;
 
 	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
 		gtk_widget_destroy (file_sel);
@@ -680,8 +489,8 @@ activate_action_open (GtkAction *action,
 
 	filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (filter, _("All archives"));
-	for (i = 0; open_mime_type[i] != NULL; i++)
-		gtk_file_filter_add_mime_type (filter, open_mime_type[i]);
+	for (i = 0; open_type[i] != FR_FILE_TYPE_NULL; i++)
+		gtk_file_filter_add_mime_type (filter, file_type_desc[open_type[i]].mime_type);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_sel), filter);
 
@@ -721,12 +530,11 @@ save_file_destroy_cb (GtkWidget *w,
 
 static void 
 save_file_response_cb (GtkWidget *w,
-		       gint response,
+		       gint       response,
 		       GtkWidget *file_sel)
 {
 	FRWindow *window;
 	char     *path;
-	char     *dir;
 
 	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
 		gtk_widget_destroy (file_sel);
@@ -735,108 +543,12 @@ save_file_response_cb (GtkWidget *w,
 
 	window = g_object_get_data (G_OBJECT (file_sel), "fr_window");
 
-	path = get_full_path (file_sel);
-
-	if ((path == NULL) || (*path == 0)) {
-		GtkWidget *dialog;
-
+	path = get_archive_filename_from_selector (window, file_sel);
+	if (path != NULL) {
+		window_archive_save_as (window, path);
+		gtk_widget_destroy (file_sel);
 		g_free (path);
-
-		dialog = _gtk_message_dialog_new (GTK_WINDOW (window->app),
-						  GTK_DIALOG_DESTROY_WITH_PARENT,
-						  GTK_STOCK_DIALOG_ERROR,
-						  _("Could not save the archive"),
-						  _("You have to specify an archive name."),
-						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
-						  NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-		return;
 	}
-	
-	dir = remove_level_from_path (path);
-	if (access (dir, R_OK | W_OK | X_OK) != 0) {
-		GtkWidget *dialog;
-
-		g_free (dir);
-		g_free (path);
-
-		dialog = _gtk_message_dialog_new (GTK_WINDOW (window->app),
-						  GTK_DIALOG_DESTROY_WITH_PARENT,
-						  GTK_STOCK_DIALOG_ERROR,
-						  _("Could not save the archive"),
-						  _("You don't have permission to create an archive in this folder"),
-						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
-						  NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-		return;
-	}
-	g_free (dir);
-
-#ifdef DEBUG
-	g_print ("save as %s\n", path); 
-#endif
-
-	if (path_is_file (path)) {
-		GtkWidget *dialog;
-		int r;
-
-		if (!is_supported_extension(file_sel, path)) {
-                        dialog = _gtk_message_dialog_new (GTK_WINDOW (file_sel),
-                                                  GTK_DIALOG_MODAL,
-                                                  GTK_STOCK_DIALOG_ERROR,
-                                                  _("Could not create the archive"),
-                                                  _("Archive type not supported."),
-                                                  GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
-                                                  NULL);
-                        gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
-                        gtk_dialog_run (GTK_DIALOG (dialog));
-                        gtk_widget_destroy (GTK_WIDGET (dialog));
-                        g_free (path);
-                        return;
-                }
-
-		dialog = _gtk_message_dialog_new (GTK_WINDOW (window->app),
-						  GTK_DIALOG_MODAL,
-						  GTK_STOCK_DIALOG_QUESTION,
-						  _("The archive already exists.  Do you want to overwrite it?"),
-						  NULL,
-						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						  _("Overwrite"), GTK_RESPONSE_YES,
-						  NULL);
-
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
-		r = gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-
-		if (r != GTK_RESPONSE_YES) {
-			g_free (path);
-			return;
-		}
-
-		if (unlink (path) != 0) {
-			GtkWidget *dialog;
-			dialog = _gtk_message_dialog_new (GTK_WINDOW (window->app),
-							  GTK_DIALOG_DESTROY_WITH_PARENT,
-							  GTK_STOCK_DIALOG_ERROR,
-							  _("Could not delete the old archive."),
-							  NULL,
-							  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
-							  NULL);
-			gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-			gtk_dialog_run (GTK_DIALOG (dialog));
-			gtk_widget_destroy (GTK_WIDGET (dialog));
-			g_free (path);
-			return;
-		}
-	} 
-
-	window_archive_save_as (window, path);
-	gtk_widget_destroy (file_sel);
-	g_free (path);
 }
 
 
@@ -844,12 +556,12 @@ void
 activate_action_save_as (GtkAction *action, 
 			 gpointer   data)
 {
-	FRWindow  *window = data;
-	GtkWidget *file_sel;
-	GtkWidget *hbox;
-	GtkWidget *combo_box;
+	FRWindow      *window = data;
+	GtkWidget     *file_sel;
+	GtkWidget     *hbox;
+	GtkWidget     *combo_box;
 	GtkFileFilter *filter;
-	int i;
+	int            i;
 
 	file_sel = gtk_file_chooser_dialog_new (
 			_("Save"),
@@ -863,10 +575,13 @@ activate_action_save_as (GtkAction *action,
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_sel),
 					     window->open_default_dir);
 
+	if (window->archive_filename != NULL) 
+		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_sel), file_name_from_path (window->archive_filename));
+
 	filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (filter, _("All archives"));
-	for (i = 0; save_mime_type[i] != NULL; i++)
-		gtk_file_filter_add_mime_type (filter, save_mime_type[i]);
+	for (i = 0; save_type[i] != FR_FILE_TYPE_NULL; i++)
+		gtk_file_filter_add_mime_type (filter, file_type_desc[save_type[i]].mime_type);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_sel), filter);
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_sel), filter);
 
@@ -883,14 +598,16 @@ activate_action_save_as (GtkAction *action,
 			    FALSE, FALSE, 0);
 	
 	combo_box = gtk_combo_box_new_text ();
-	for (i = 0; write_type_desc[i].name != NULL; i++)
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), _("Automatic"));
+	for (i = 0; save_type[i] != FR_FILE_TYPE_NULL; i++)
 		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
-					   _(write_type_desc[i].name));
+					   _(file_type_desc[save_type[i]].name));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
 	gtk_box_pack_start (GTK_BOX (hbox), combo_box, TRUE, TRUE, 0);
 	gtk_widget_show_all (hbox);
 
-	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel), hbox);	
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel), hbox);
+	
 	/**/
 
 	g_object_set_data (G_OBJECT (file_sel), "fr_window", window);
@@ -900,11 +617,14 @@ activate_action_save_as (GtkAction *action,
 			  "response", 
 			  G_CALLBACK (save_file_response_cb), 
 			  file_sel);
-
 	g_signal_connect (G_OBJECT (file_sel),
 			  "destroy", 
 			  G_CALLBACK (save_file_destroy_cb),
 			  file_sel);
+	g_signal_connect (G_OBJECT (combo_box),
+			  "changed", 
+			  G_CALLBACK (filetype_combobox_changed_cb),
+			  file_sel);
 
 	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
 	gtk_widget_show_all (file_sel);
@@ -1046,8 +766,7 @@ activate_action_view_or_open (GtkAction *action,
 	file_list = window_get_file_list_selection (window, FALSE, NULL);
 	if (file_list == NULL) 
 		return;
-
-	window_view_or_open_file (window, (gchar*) file_list->data);
+	window_view_or_open_file (window, (char*) file_list->data);
 	path_list_free (file_list);	
 }
 
